const ejs = require('ejs')
const chalk = require('chalk')
const fs = require('fs-extra')
const symbols = require('log-symbols')
const { promisify } = require('util')
const figlet = promisify(require('figlet'))
const path = require('path')
const inquirer = require('inquirer')
const { log } = require('../util/log')
const configPath = path.resolve(__dirname,'../config.json')

const map = {
  list: {
    'vue': {
      temp: 'list.vue.ejs',
      ext: 'vue',
    },
    'js': {
      temp: 'list.service.js.ejs',
      ext: 'service.js',
    }
  },
  report: {
    'vue': {
      temp: 'report.vue.ejs',
      ext: 'vue',
    },
    'js': {
      temp: 'report.model.js.ejs',
      ext: 'model.js',
    }
  }
}

module.exports = async (dirname, options) => {
  let template, tempName
  // 如果有传参则使用传入的模板
  const { temp } = options
  if (temp in map) {
    tempName = temp
    template = map[temp]
    const fileName = dirname.split('/')
      .map(str => str.slice(0, 1).toUpperCase() + str.slice(1).toLowerCase())
      .join('')
    const jsonConfig = await fs.readJson(configPath)
    try {
      for (const [key, value] of Object.entries(template)) {
        if (key) {
          const {temp, ext} = value
          const baseUrl = jsonConfig[`${tempName}${key}`]
          const filepath = `${baseUrl}/${dirname}`
          await compile(filepath, temp, {
            name: fileName,
            ext,
            ...options,
          })
        }
      }
  
      log.default(symbols.success, chalk.cyan('🚀 generate success'))
      const info = await figlet('yyds', {
        font: 'Ghost',
        horizontalLayout: 'default',
        verticalLayout: 'default',
        width: 100,
        whitespaceBreak: true
      })
      log.green(info)
    } catch (err) {
      log.default(symbols.error, chalk.red(`Generate failed. ${err}`))
    }
  } else {
    log.default(symbols.error, chalk.red(`Sorry, don't support this template`))
  }
}

/**
 * 编译模板生成结果
*/
const compile = async (filepath, templatepath, options) => {
  const cwd = process.cwd();
  const targetDir = path.resolve(cwd,`${filepath}.${options.ext}`);
  const tempDir = path.resolve(__dirname, `../template/${templatepath}`);
  if (fs.existsSync(tempDir)) {
    if (fs.existsSync(targetDir)) {
      if (options.force) {
        fs.remove(targetDir)
      } else {
        const { action } = await inquirer.prompt([
          {
            name: 'action',
            type: 'list',
            message: `Target directory ${chalk.cyan(targetDir)} already exists. Pick an action:`,
            choices: [
              { name: 'Overwrite', value: 'overwrite' },
              { name: 'Cancel', value: false }
            ]
          }
        ])
        if (!action) {
          return
        } else if (action === 'overwrite') {
          log.default(`👻Removing ${chalk.cyan(targetDir)}`)
          await fs.remove(targetDir)
        }
      }
    }
    const content = fs.readFileSync(tempDir).toString()
    const result = ejs.compile(content)(options)
    // fs.writeFileSync(`${targetDir}`, result)
    await writeFileEnsure(filepath, result, options.ext)
  } else {
    throw Error("Don't find target template in  directory")
    // log.default(symbols.error, chalk.red("Don't find target template in  directory"))
  }
}

/**
 *
 * @param {*} paths 路径
 * @param {*} data 写入的数据
 * @param {*} ext 文件后缀
 */
function writeFileEnsure(paths, data, ext) {
  const cwd = process.cwd();
  const pathArr = paths.split('/')
  if (pathArr.length > 1) {
    pathArr.reduce((prev, cur, index) => {
      const baseUrl = path.resolve(cwd, `${prev}/${cur}`)
      if (index === pathArr.length - 1) {
        fs.writeFileSync(path.resolve(cwd, `${baseUrl}.${ext}`), data)
      } else {
        if (!fs.existsSync(baseUrl)) {
          fs.mkdirSync(path.resolve(cwd, baseUrl))
        }
      }
      return prev + '/' + cur
    }, '.')
  } else {
    fs.writeFileSync(path.resolve(cwd, `${paths}.${ext}`), data)
  }
}
